"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bootstrap = exports.BootstrapStarter = exports.isTypeScriptEnvironment = void 0;
const path_1 = require("path");
const logger_1 = require("@midwayjs/logger");
function isTypeScriptEnvironment() {
    const TS_MODE_PROCESS_FLAG = process.env.MIDWAY_TS_MODE;
    if ('false' === TS_MODE_PROCESS_FLAG) {
        return false;
    }
    // eslint-disable-next-line node/no-deprecated-api
    return TS_MODE_PROCESS_FLAG === 'true' || !!require.extensions['.ts'];
}
exports.isTypeScriptEnvironment = isTypeScriptEnvironment;
class BootstrapStarter {
    constructor() {
        this.bootstrapItems = [];
        this.globalOptions = {};
    }
    configure(options) {
        this.globalOptions = options;
        return this;
    }
    load(unit) {
        this.bootstrapItems.push(unit);
        return this;
    }
    async init() {
        this.appDir = this.globalOptions.baseDir || process.cwd();
        await Promise.all(this.getActions('initialize', {
            ...this.globalOptions,
            baseDir: this.getBaseDir(),
            appDir: this.appDir,
        }));
    }
    async run() {
        await Promise.all(this.getActions('run', {}));
    }
    async stop() {
        await Promise.all(this.getActions('stop', {}));
    }
    getActions(action, args) {
        return this.bootstrapItems.map(item => {
            return item[action](args);
        });
    }
    getBaseDir() {
        if (isTypeScriptEnvironment()) {
            return path_1.join(this.appDir, 'src');
        }
        else {
            return path_1.join(this.appDir, 'dist');
        }
    }
}
exports.BootstrapStarter = BootstrapStarter;
class Bootstrap {
    /**
     * set global configuration for midway
     * @param configuration
     */
    static configure(configuration) {
        var _a;
        if (!this.logger && !configuration.logger) {
            this.logger = logger_1.createConsoleLogger('bootstrapConsole');
            if (configuration.logger === false) {
                (_a = this.logger) === null || _a === void 0 ? void 0 : _a.disableConsole();
            }
            configuration.logger = this.logger;
        }
        this.getStarter().configure(configuration);
        return this;
    }
    /**
     * load midway framework unit
     * @param unit
     */
    static load(unit) {
        this.getStarter().load(unit);
        return this;
    }
    static getStarter() {
        if (!this.starter) {
            this.starter = new BootstrapStarter();
        }
        return this.starter;
    }
    static async run() {
        // https://nodejs.org/api/process.html#process_signal_events
        // https://en.wikipedia.org/wiki/Unix_signal
        // kill(2) Ctrl-C
        process.once('SIGINT', this.onSignal.bind(this, 'SIGINT'));
        // kill(3) Ctrl-\
        process.once('SIGQUIT', this.onSignal.bind(this, 'SIGQUIT'));
        // kill(15) default
        process.once('SIGTERM', this.onSignal.bind(this, 'SIGTERM'));
        process.once('exit', this.onExit.bind(this));
        await this.getStarter().init();
        return this.getStarter()
            .run()
            .then(() => {
            this.logger.info('[midway:bootstrap] current app started');
        })
            .catch(err => {
            this.logger.error(err);
            process.exit(1);
        });
    }
    static async stop() {
        return this.getStarter().stop();
    }
    /**
     * on bootstrap receive a exit signal
     * @param signal
     */
    static async onSignal(signal) {
        this.logger.info('[midway:bootstrap] receive signal %s, closing', signal);
        try {
            await this.stop();
            this.logger.info('[midway:bootstrap] close done, exiting with code:0');
            process.exit(0);
        }
        catch (err) {
            this.logger.error('[midway:bootstrap] close with error: ', err);
            process.exit(1);
        }
    }
    /**
     * on bootstrap process exit
     * @param code
     */
    static onExit(code) {
        this.logger.info('[midway:bootstrap] exit with code:%s', code);
    }
}
exports.Bootstrap = Bootstrap;
//# sourceMappingURL=bootstrap.js.map